<?php

/**
	ARA Class
	PHP RSS Builder
	(c) 2007-2009 Denis Sureau
	http://www.scriptol.com/rss/
	
	PHP API for creating RSS 2.0 feeds
	Licence GNU GPL 2.

  The ARA RSS class has methods to 
  - read or 
  - create a feed,
  - build a channel, 
  - add items, 
  - delete items, 
  - store the feed.
*/  

class ara
{
	public $doc = "";
	public $rss = "";
	public $channel = "";
	
	public $DESC_SIZE = 240;   // Max length of a description
	public $FEED_MAX = 15;    // Max number of items
	public $DESCENDING = true; // Add item at top if true, at end else

	function makeTag($tagname, $str)
	{
		$tag= $this->doc->createElement($tagname);
		$data= $this->doc->createTextNode($str);
		$tag->appendChild($data);
	
		return $tag;
	}
	
	// Retrieve title and desc from a page.
	// Input: the URL of the page.
	// Ouput: the data plus the date.
	
	function getData($url)
	{
	
    $metas = get_meta_tags($url);
    $desc = $metas["description"];

    $home = file_get_contents($url);
    
    preg_match("/<title>(.)*<\/title>/i", $home, $res);
    $x = $res[0];
    $a = strpos($x, '>') + 1;
    $b = strrpos($x, '<');
    $title = substr($x , $a, $b - $a);

    if($title == "")
    {
      preg_match("/<h1(.)*>(.)*<\/h1>/i", $home, $res);
      $x = $res[0];
      $a = strpos($x, '>') + 1;
      $b = strrpos($x, '<');
      $title = substr($x , $a, $b - $a);
    }    

    if($desc == "")
    {
      preg_match_all("/<p(.)*>(.)*<\/p>/i" , $home, $res, PREG_PATTERN_ORDER);
      for($i = 0; $i < count($res[0]); $i++)
      {
        $x = $res[0][$i];
        $a = strpos($x, '>') + 1;
        $b = strrpos($x, '<');
        $desc .= substr($x , $a, $b - $a);
        if(strlen($desc) >= $this->DESC_SIZE) break;
      }  
      $r = strrpos($desc, ".");
      if($r > ($this->DESC_SIZE /2))
        $desc = substr($desc, 0, $r);
    }    
    
  
    $date = date(DATE_RFC822);
    
    return array($title, $desc, $date); 
  }

  // Build a feed file

	public function ARAFeed($title, $link, $desc, $date)
	{
		$this->doc = new DOMDocument("1.0");
		$this->doc->formatOutput = true;
	
		$this->rss = $this->doc->createElement("rss");
		$this->rss->setAttribute("version", "2.0");
		$this->doc->appendChild($this->rss);
	
		$this->channel = $this->doc->createElement("channel");
		$this->rss->appendChild($this->channel);
	
		$this->channel->appendChild($this->makeTag("title", $title));
		$this->channel->appendChild($this->makeTag("link", $link));
		$this->channel->appendChild($this->makeTag("description", $desc));
		$this->channel->appendChild($this->makeTag("pubDate", $date));
	
	}

  // Add an item to the feed

	public function ARAItem( $title, $link, $desc, $date)
	{
		$item = $this->doc->createElement("item");
	
		$item->appendChild($this->makeTag("title", $title));	
		$item->appendChild($this->makeTag("link", $link));	
		$item->appendChild($this->makeTag("description", $desc));	
		$item->appendChild($this->makeTag("pubDate", $date));	
	
	  $item = $this->channel->appendChild($item);  // Add it to end in any case 
	  if(!$this->DESCENDING) return;

    // Moving before first item
    $dnl = $this->channel->getElementsByTagName("item");
    if($dnl->length == 1) return;
    $first = $dnl->item(0);
    if($first != null) // assumed to be an item
    {
       $this->channel->insertBefore($item, $first);  
    } 
    else
      die("Error in AAItem");  
	}
	
  // Delete last items to a limit. By default the limit is FEED_MAX.
  // May be used to free the channel.

	public function compress($tosize = -1)
	{
    $dnl = $this->channel->getElementsByTagName("item");
    $size = $dnl->length;
    
    if($tosize < 0) $tosize = $this->FEED_MAX;
    
    if(!$this->DESCENDING) // not descending remove at top
    {
      while($size > $tosize)
      {
        $this->channel->removeChild($dnl->item(0));
        $size--;
      }
      return;  
    }
 
    while($size > $tosize)  // remove at end
    {
      $size--;
      $this->channel->removeChild($dnl->item($size));
    }
	}
  
  // Delete a specified item
  
  public function remove($item)
  {
    $this->channel->removeChild($item);
  }
  	
	
	// For a new feed, create the channel tag
	// Input: the url of the home page
	
	public function build($url)
	{
 
    $domain = $_SERVER['HTTP_HOST'];
    $link = "http://".$domain;  // Home page
    
    list($title, $desc, $date) = $this->getData($url);
 
    $this->ARAFeed($title, $link, $desc, $date);
  }
	
	// Load an existing RSS file
	// Output: true if it has a channel, false otherwise.
	
	public function load($fname)
	{
		if(!file_exists($fname)) return false;

		$this->doc = new DOMDocument("1.0");
		$this->doc->formatOutput = true;

    @$this->doc->load($fname);
        
    @$dnl = $this->doc->getElementsByTagName("channel");
    if($dnl->length == 0) return false;
      
    $this->channel = $dnl->item(0);
    return true;
  }
	
	// Save into a file
	// Returns the number of bytes written of false
	
	public function save($fname)
	{
		return $this->doc->save($fname);
	}
	
	// Convert to a string
	
	public function saveXML()
	{
		return $this->doc->saveXML();
	}

}


?>
